@@ -1,4 +1,4 @@
-//@line 40 "/mnt/data/212/toolkit/components/viewconfig/content/config.js"
+//@line 41 "/home/gscrivano/icecat/302/toolkit/components/viewconfig/content/config.js"
 
 const nsIPrefLocalizedString = Components.interfaces.nsIPrefLocalizedString;
 const nsISupportsString = Components.interfaces.nsISupportsString;
@@ -118,6 +118,7 @@ var view = {
   selectionChanged : function() {},
   cycleCell: function(row, col) {},
   isEditable: function(row, col) {return false; },
+  isSelectable: function(row, col) {return false; },
   setCellValue: function(row, col, value) {},
   setCellText: function(row, col, value) {},
   performAction: function(action) {},
@@ -283,6 +284,17 @@ function onConfigLoad()
   gTypeStrs[nsIPrefBranch.PREF_INT] = gConfigBundle.getString("int");
   gTypeStrs[nsIPrefBranch.PREF_BOOL] = gConfigBundle.getString("bool");
 
+  var showWarning = gPrefBranch.getBoolPref("general.warnOnAboutConfig");
+
+  if (showWarning)
+    document.getElementById("warningButton").focus();
+  else
+    ShowPrefs();
+}
+
+// Unhide the warning message
+function ShowPrefs()
+{
   var prefCount = { value: 0 };
   var prefArray = gPrefBranch.getChildList("", prefCount);
 
@@ -313,33 +325,58 @@ function onConfigLoad()
   
   gPrefBranch.addObserver("", gPrefListener, false);
 
-  document.getElementById("configTree").view = view;
-  
+  var configTree = document.getElementById("configTree");
+  configTree.view = view;
+  configTree.controllers.insertControllerAt(0, configController);
+
+  document.getElementById("configDeck").setAttribute("selectedIndex", 1);
+  if (!document.getElementById("showWarningNextTime").checked)
+    gPrefBranch.setBoolPref("general.warnOnAboutConfig", false);
+
   document.getElementById("textbox").focus();
 }
 
 function onConfigUnload()
 {
-  gPrefBranch.removeObserver("", gPrefListener);
-  document.getElementById("configTree").view = null;
+  if (document.getElementById("configDeck").getAttribute("selectedIndex") == 1) {
+    gPrefBranch.removeObserver("", gPrefListener);
+    var configTree = document.getElementById("configTree");
+    configTree.view = null;
+    configTree.controllers.removeController(configController);
+  }
 }
 
 function FilterPrefs()
 {
-  var substring = document.getElementById("textbox").value.toLowerCase();
+  var substring = document.getElementById("textbox").value;
+  var rex;
+  // Check for "/regex/[i]"
+  if (substring.charAt(0) == '/') {
+    var r = substring.match(/^\/(.*)\/(i?)$/);
+    try {
+      rex = RegExp(r[1], r[2]);
+    }
+    catch (e) {
+      return; // Do nothing on incomplete or bad RegExp
+    }
+  }
+
   var prefCol = view.selection.currentIndex < 0 ? null : gPrefView[view.selection.currentIndex].prefCol;
-  var array = gPrefView;
+  var oldlen = gPrefView.length;
   gPrefView = gPrefArray;
   if (substring) {
     gPrefView = [];
+    if (!rex)
+      rex = RegExp(substring.replace(/([^* \w])/g, "\\$1").replace(/^\*+/, "")
+                            .replace(/\*+/g, ".*"), "i");
     for (var i = 0; i < gPrefArray.length; ++i)
-      if (gPrefArray[i].prefCol.toLowerCase().indexOf(substring) >= 0)
+      if (rex.test(gPrefArray[i].prefCol + ";" + gPrefArray[i].valueCol))
         gPrefView.push(gPrefArray[i]);
     if (gFastIndex < gPrefArray.length)
       gPrefView.sort(gSortFunction);
   }
   view.treebox.invalidate();
-  view.treebox.rowCountChanged(array.length, gPrefView.length - array.length);
+  view.treebox.rowCountChanged(oldlen, gPrefView.length - oldlen);
   gotoPref(prefCol);
   document.getElementById("button").disabled = !substring;
 }
@@ -392,14 +429,29 @@ const gSortFunctions =
   valueCol: valueColSortFunction
 };
 
+const configController = {
+  supportsCommand: function supportsCommand(command) {
+    return command == "cmd_copy";
+  },
+  isCommandEnabled: function isCommandEnabled(command) {
+    return view.selection && view.selection.currentIndex >= 0;
+  },
+  doCommand: function doCommand(command) {
+    copyPref();
+  },
+  onEvent: function onEvent(event) {
+  }
+}
+
 function updateContextMenu()
 {
   var lockCol = PREF_IS_LOCKED;
   var typeCol = nsIPrefBranch.PREF_STRING;
   var valueCol = "";
   var copyDisabled = true;
+  var prefSelected = view.selection.currentIndex >= 0;
 
-  if (view.selection.currentIndex >= 0) {
+  if (prefSelected) {
     var prefRow = gPrefView[view.selection.currentIndex];
     lockCol = prefRow.lockCol;
     typeCol = prefRow.typeCol;
@@ -407,6 +459,9 @@ function updateContextMenu()
     copyDisabled = false;
   }
 
+  var copyPref = document.getElementById("copyPref");
+  copyPref.setAttribute("disabled", copyDisabled);
+
   var copyName = document.getElementById("copyName");
   copyName.setAttribute("disabled", copyDisabled);
 
@@ -417,23 +472,22 @@ function updateContextMenu()
   resetSelected.setAttribute("disabled", lockCol != PREF_IS_USER_SET);
 
   var canToggle = typeCol == nsIPrefBranch.PREF_BOOL && valueCol != "";
+  // indicates that a pref is locked or no pref is selected at all
+  var isLocked = lockCol == PREF_IS_LOCKED;
 
   var modifySelected = document.getElementById("modifySelected");
-  modifySelected.setAttribute("disabled", lockCol == PREF_IS_LOCKED);
+  modifySelected.setAttribute("disabled", isLocked);
   modifySelected.hidden = canToggle;
 
   var toggleSelected = document.getElementById("toggleSelected");
-  toggleSelected.setAttribute("disabled", lockCol == PREF_IS_LOCKED);
+  toggleSelected.setAttribute("disabled", isLocked);
   toggleSelected.hidden = !canToggle;
-  
-  var entry = gPrefView[view.selection.currentIndex];
-  var isLocked = gPrefBranch.prefIsLocked(entry.prefCol);
+}
 
-  // These might not exist (bug 289136)
-  try {
-    document.getElementById("lockSelected").hidden = isLocked;
-    document.getElementById("unlockSelected").hidden = !isLocked;
-  } catch (ex) {}
+function copyPref()
+{
+  var pref = gPrefView[view.selection.currentIndex];
+  gClipboardHelper.copyString(pref.prefCol + ';' + pref.valueCol);
 }
 
 function copyName()
@@ -501,39 +555,28 @@ function ModifyPref(entry)
     if (!entry.valueCol && !gPromptService.select(window, title, entry.prefCol, 2, [false, true], check))
       return false;
     gPrefBranch.setBoolPref(entry.prefCol, check.value);
-  } else {
+  }
+  else if (entry.typeCol == nsIPrefBranch.PREF_INT) {
+    var params = { windowTitle: title,
+                   label: entry.prefCol,
+                   value: entry.valueCol,
+                   cancelled: true };
+    window.openDialog("chrome://global/content/configIntValue.xul", "_blank",
+                      "chrome,titlebar,centerscreen,modal", params);
+    if (params.cancelled)
+      return false;
+    gPrefBranch.setIntPref(entry.prefCol, params.value);
+  }
+  else {
     var result = { value: entry.valueCol };
     var dummy = { value: 0 };
     if (!gPromptService.prompt(window, title, entry.prefCol, result, null, dummy))
       return false;
-    if (entry.typeCol == nsIPrefBranch.PREF_INT) {
-      gPrefBranch.setIntPref(entry.prefCol, parseInt(result.value, 10));
-    } else {
-      var supportsString = Components.classes[nsSupportsString_CONTRACTID].createInstance(nsISupportsString);
-      supportsString.data = result.value;
-      gPrefBranch.setComplexValue(entry.prefCol, nsISupportsString, supportsString);
-    }
+    var supportsString = Components.classes[nsSupportsString_CONTRACTID].createInstance(nsISupportsString);
+    supportsString.data = result.value;
+    gPrefBranch.setComplexValue(entry.prefCol, nsISupportsString, supportsString);
   }
-  
-  gPrefService.savePrefFile(null);
-  
-  // Fire event for accessibility
-  var event = document.createEvent('Events');
-  event.initEvent('NameChange', false, true);
-  document.getElementById("configTree").dispatchEvent(event);
 
+  gPrefService.savePrefFile(null);
   return true;
 }
-
-function LockSelected()
-{
-  var entry = gPrefView[view.selection.currentIndex];
-  gPrefBranch.lockPref(entry.prefCol);
-}
-
-function UnlockSelected()
-{
-  var entry = gPrefView[view.selection.currentIndex];
-  gPrefBranch.unlockPref(entry.prefCol);
-}
-
